#include <stdio.h>
#include <math.h>
#include <fstream>
#include <iostream>
#include <stdlib.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_fft_complex.h>

/*
This program performs a 1/f^p filtering of the frequency 
followed by a 2D Fast Fourier Transform. The initial Fourier 
Coefficients must be given in an external file
*/

#define REAL(z,i) ((z)[2*(j)])
#define IMAG(z,i) ((z)[2*(j)+1])

const int n = 128;
const int reduct = 1; /* All data must be written out. */

typedef struct{
  double all_data[n][n][2];
  double data[2*n];
} FourierStruct;

FourierStruct* const tmp_fourier = new FourierStruct;

int main(int argc, char* argv[]){
  int tmp_x, tmp_y; 
  float decay_pow, central_freq;

  if(argc < 4){
    std::cout << "Usage: ./Freq_Filter_FFT_2D.srl ";
    std::cout << "infile outfile central_freq power " << std::endl;
    return(0);
  }
  
  central_freq = atof(argv[3]);
  decay_pow    = atof(argv[4]);
  /****************** Read the input data *******************/
  std::ifstream fin;
  fin.open(argv[1]);
  if (fin.is_open()){
    for (int i = 0; i < n; i++){
      for (int j = 0; j < n; j++){
	
	fin >> tmp_x;
	fin >> tmp_y;
	fin >> tmp_fourier->all_data[tmp_x][tmp_y][0];
	fin >> tmp_fourier->all_data[tmp_x][tmp_y][1];
	
	tmp_fourier->all_data[tmp_x][tmp_y][0] = tmp_fourier->all_data[tmp_x][tmp_y][0]/
	  pow(sqrt(pow(tmp_x - central_freq-0.5, 2) + pow(tmp_y - central_freq-0.5, 2)), decay_pow);
	tmp_fourier->all_data[tmp_x][tmp_y][1] = tmp_fourier->all_data[tmp_x][tmp_y][1]/
	  pow(sqrt(pow(tmp_x - central_freq-0.5, 2) + pow(tmp_y - central_freq-0.5, 2)), decay_pow);
      }
    }
  }
  else{
    std::cout << " Could not open " << argv[1] << std::endl; 
    return(-1);
  }
  fin.close();
  /**********************************************************/

  gsl_fft_complex_wavetable* wavetable; /* Pointers to structures. */
  gsl_fft_complex_workspace* workspace;
  
  wavetable = gsl_fft_complex_wavetable_alloc (n);
  workspace = gsl_fft_complex_workspace_alloc (n);
  
  /****************** Perform N FFTs along the Y axis *******************/
  for(int i = 0; i < n; ++i){
    for (int j = 0; j < n; j++){
      REAL(tmp_fourier->data, j) = tmp_fourier->all_data[i][j][0];
      IMAG(tmp_fourier->data, j) = tmp_fourier->all_data[i][j][1];
    }
    gsl_fft_complex_forward(tmp_fourier->data, 1, n, wavetable, workspace);
    
    for (int j = 0; j < n; j++){
      tmp_fourier->all_data[i][j][0] = REAL(tmp_fourier->data, j);
      tmp_fourier->all_data[i][j][1] = IMAG(tmp_fourier->data, j);
    }
  }
  /****************** Perform N FFTs along the X axis *******************/
  for(int i = 0; i < n; ++i){
    for (int j = 0; j < n; j++){
      REAL(tmp_fourier->data, j) = tmp_fourier->all_data[j][i][0];
      IMAG(tmp_fourier->data, j) = tmp_fourier->all_data[j][i][1];
    }
    gsl_fft_complex_forward (tmp_fourier->data, 1, n, wavetable, workspace);
    
    for (int j = 0; j < n; j++){
      tmp_fourier->all_data[j][i][0] = REAL(tmp_fourier->data, j);
      tmp_fourier->all_data[j][i][1] = IMAG(tmp_fourier->data, j);
    }
  }
  
  std::ofstream fout;
  fout.open(argv[2]);
  if (fout.is_open()){
    for (int i = 0; i < n; i++){
      for (int j = 0; j < n; j++){
	fout << i << " " << j << " " 
	     << tmp_fourier->all_data[i][j][0] << " " 
	     << tmp_fourier->all_data[i][j][1] << std::endl; 
      }
    }
  }
  else{
    std::cout << " Could not open " << argv[2] << std::endl; 
    return(0);
  }
  fout.close();
  
  gsl_fft_complex_wavetable_free (wavetable);
  gsl_fft_complex_workspace_free (workspace);
  
  std::cout << " Fourier Transform Done. " << std::endl;
  return 0;
}
