#! /usr/bin/python
# this is the first comment

import sys, os, math, array

class GenPovFile():

    def __init__(self, file_in, file_out):
        self.grid_points     = 128
        self.tot_points      = self.grid_points
        self.scalar_amplify  = 30.0
        self.min_y           = 1
        self.max_y           = -1
        self.max_x           = 0
        self.max_z           = 0
        self.color_map       = \
"""[0.0  color Green]
[0.125  color Blue]
[0.25  color Red]
[0.375  color Orange]
[0.5  color Sienna]
[0.625  color SteelBlue]
[0.75  color Cyan]
[0.875  color Yellow]
[1.0  color Magenta]"""
        self.nm_color_map    = \
"""[0.0  color Black]
[0.125  color Gray10]
[0.25  color Gray30]
[0.375  color Gray40]
[0.5  color Gray50]
[0.625  color Gray60]
[0.75  color Gray70]
[0.875  color Gray90]
[1.0  color White]"""      

        self.scale           = 1.0
        self.scale_y         = 1.0
        self.file_in         = file_in
        self.povrayfile      = file_out
        #self.color_map       = self.nm_color_map

    def readfile(self, FILEREAD):
        fin  = open(FILEREAD , 'r')

        self.Coords = {}
        tmp_val_list = []
        
        curr_str = 'aaa' #something not empty
        arggg = 0
        while curr_str != '' :
            curr_str = fin.readline()
            if curr_str != '' :
                curr_line = curr_str.split(' ')
                
                cx = int(curr_line[0]) 
                cy = math.sqrt(math.pow(float(curr_line[2]), 2) + math.pow(float(curr_line[3]), 2)) 
                cz = int(curr_line[1]) 

                if cx not in self.Coords.keys():
                    self.Coords[cx] = {}

                if cz not in self.Coords[cx].keys():
                    self.Coords[cx][cz] = cy
                tmp_val_list.append(cy)
        fin.close()
        tmp_val_list.sort()
        self.min_y = tmp_val_list[0]
        self.max_y = tmp_val_list[-1]

    def findMinMaxZ(self):

        self.scale_y = 1.0/(self.max_y - self.min_y)

        self.x_list = [s for s in self.Coords.keys()]
        self.x_list.sort()
        self.max_x = self.x_list[-1]
        self.z_list = [s for s in self.Coords[self.max_x].keys()]
        self.z_list.sort()
        self.max_z = self.z_list[-1]

    def drawTriangles(self):        
        outstring = """#include \"colors.inc\"
#include \"textures.inc\"
#include \"glass.inc\"
background{Black}"""

        outstring += """camera { angle 10
location <%s, %s, %s>
look_at <%s, %s, %s>}\n""" % (0, 40*self.scalar_amplify, 0, 0, 0, 0)
        
        outstring += """light_source { <%s, %s, %s> color White shadowless}
#declare surfface = union{ \n""" % (0, 40*self.scalar_amplify, 0)
        
        for x_c in self.x_list[:-1]:
            for z_c in self.z_list[:-1]:

                outstring += 'triangle{'
                outstring += '<%s, %s, %s>' % (str(x_c - self.max_x/2.0), 
                                               str(self.scale_y*(self.Coords[x_c][z_c] - self.min_y)),
                                               str(z_c - self.max_z/2.0))
                outstring += '<%s, %s, %s>' % (str(x_c + 1 - self.max_x/2.0),
                                               str(self.scale_y*(self.Coords[x_c + 1][z_c] - self.min_y)), 
                                               str(z_c - self.max_z/2.0))
                outstring += '<%s, %s, %s>\n' % (str(x_c - self.max_x/2.0), 
                                                 str(self.scale_y*(self.Coords[x_c][z_c + 1] - self.min_y)),
                                                 str(z_c + 1 - self.max_z/2.0))
                outstring += """pigment{gradient y
color_map {
%s
}}}\n""" % self.color_map
                
                outstring += 'triangle{'
                outstring += '<%s, %s, %s>' % (str(x_c + 1 - self.max_x/2.0), 
                                               str(self.scale_y*(self.Coords[x_c + 1][z_c] - self.min_y)), 
                                               str(z_c - self.max_z/2.0))
                outstring += '<%s, %s, %s>' % (str(x_c - self.max_x/2.0),
                                               str(self.scale_y*(self.Coords[x_c][z_c + 1] - self.min_y)),
                                               str(z_c + 1 - self.max_z/2.0))
                outstring += '<%s, %s, %s>\n' % (str(x_c + 1 - self.max_x/2.0),
                                                 str(self.scale_y*(self.Coords[x_c + 1][z_c + 1] - self.min_y)),
                                                 str(z_c + 1 - self.max_z/2.0))
                outstring += """pigment{gradient y
color_map {
%s
}}}\n""" % self.color_map
        outstring +='}\n'
        outstring += 'object{surfface scale<1.33, 1, 1>}\n'
        f  = open(self.povrayfile , 'w')
        f.write(outstring)
        f.close()

    def execute(self):
        self.readfile(self.file_in)
        self.findMinMaxZ()
        self.drawTriangles()
        #os.system('povray %s +O./ %s > /dev/null 2>&1' % (self.povrayfile, self.file_out))
        #os.system('povray %s +O./%s' % (self.povrayfile, self.file_out))

def main():
    file_in  = sys.argv[1]
    povray_file = sys.argv[2]

    X = GenPovFile(file_in, povray_file)
    X.execute()

if __name__ == "__main__":
    main()
